/*
 * Copyright (c) 2012 Samsung Electronics Co., Ltd. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/*global document, window, tizen, console*/

// strict wrapper

/**
 * App implementation.
 * @namespace Compass/main
 * @memberof Compass
 */
(function strict() {
    'use strict';

    /**
     * Direction HTML element.
     * @type {HTMLElement}
     */
    var directionEl = null,

        /**
         * Angle HTML element.
         * @type {HTMLElement}
         * @memberof Compass/main
         */
        angleEl = null,

        /**
         * Rotation HTML element.
         * @type {HTMLElement}
         * @memberof Compass/main
         */
        rotationEl = null,

        /**
         * Compass rotation value.
         * @type {number}
         * @memberof Compass/main
         */
        rotation = 0;

    /**
     * Obtains angle from rotation.
     * @param {number} value
     * @return {number}
     * @memberof Compass/main
     */
    function angleFromRotation(value) {
        var angle = -value % 360;

        if (angle < 0) {
            angle += 360;
        }
        return angle;
    }

    /**
     * Handles device orientation data.
     *
     * @param {DeviceOrientationEvent} eventData Device orientation event.
     * @memberof Compass/main
     */
    function onDeviceOrientation(eventData) {
        var angle = eventData.alpha,
            lastAngle = angleFromRotation(rotation),
            deltaAngle = 0,
            text = '';

        // Check if the angle has changed since the last measurement.
        if (isNaN(angle) || angle === lastAngle) {
            return;
        }

        // find direction from given angle
        if (angle < 68 || angle > 292) {
            text = 'NORTH';
        } else if (angle > 112 && angle < 248) {
            text = 'SOUTH';
        }
        if (angle > 22 && angle < 158) {
            text += 'EAST';
        } else if (angle > 202 && angle < 338) {
            text += 'WEST';
        }

        // Calculate the rotation of the compass.
        deltaAngle = lastAngle - angle;
        if (deltaAngle > 180) {
            rotation -= 360 - deltaAngle;
        } else if (deltaAngle < -180) {
            rotation += 360 + deltaAngle;
        } else {
            rotation += deltaAngle;
        }

        // Update the direction and angle labels.
        directionEl.innerHTML = text;
        angleEl.innerHTML =
            Math.round(angle) + '<sup>o</sup>';
        // Rotate the compass image.
        rotationEl.style['-webkit-transform'] =
            'rotate(' + rotation + 'deg)';
    }

    /**
     * Exits the application.
     * @memberof Compass/main
     */
    function exit() {
        try {
            tizen.application.getCurrentApplication().exit();
        } catch (err) {
            console.error('Error: ', err);
        }
    }

    /**
     * Binds events.
     * @memberof Compass/main
     */
    function bindEvents() {
        window.addEventListener('tizenhwkey', function onTizenHWKey(e) {
            if (e.keyName === 'back') {
                exit();
            }
        });
        window.addEventListener(
            'deviceorientation',
            onDeviceOrientation,
            false
        );
    }

    /**
     * Initializes the application.
     * @memberof Compass/main
     */
    function init() {
        directionEl = document.getElementById('direction');
        angleEl = document.getElementById('angle');
        rotationEl = document.getElementById('rotation');

        bindEvents();
    }

    // initialize application when the document is ready
    window.addEventListener('DOMContentLoaded', init);

}());
